/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TEXTURE_HPP_
#define _TEXTURE_HPP_

#include <iostream>
#include <vector>


namespace cpe
{
    /** \brief Container for texture */
    class texture
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief emtpy constructor */
        texture();


        // ********************************************* //
        // ********************************************* //
        //  Load/Save texture file
        // ********************************************* //
        // ********************************************* //

        /** \brief load texture file */
        void load_picture(const std::string& filename);
        /** \brief export ppm texture file */
        void export_ppm(const std::string& filename) const;

        // ********************************************* //
        // ********************************************* //
        //  Get parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get image width */
        const unsigned int& get_Nx() const;
        /** \brief get image height */
        const unsigned int& get_Ny() const;
        /** \brief get image depth */
        const unsigned int& get_depth() const;


        // ********************************************* //
        // ********************************************* //
        //  Get data
        // ********************************************* //
        // ********************************************* //

        /** \brief get internal data */
        const std::vector<unsigned char>& get_data() const;

    private:

        /** \brief load ppm texture file */
        void load_ppm_picture(const std::string& filename);

        /** \brief internal width */
        unsigned int N_x;
        /** \brief internal height */
        unsigned int N_y;
        /** \brief internal depth */
        unsigned int depth;

        /** \brief internal storage of picture data */
        std::vector<unsigned char> picture;

    };
}

#endif
