/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _EVALUATOR_SPLINE_HPP_
#define _EVALUATOR_SPLINE_HPP_

#include <matrix4.hpp>


namespace cpe
{
  /** Functor class to Evaluate spline */
  class evaluator_spline
  {
      public:

          // ********************************************* //
          // ********************************************* //
          //  Constructor
          // ********************************************* //
          // ********************************************* //

          /** \brief Direct constructor given a patch */
          evaluator_spline(const cpe::matrix4& P);

          // ********************************************* //
          // ********************************************* //
          //  Evaluation
          // ********************************************* //
          // ********************************************* //

          /** \brief Evaluation of the value of the interpolation at the (u,v) parameters */
          double operator()(const double& u,const double& v) const;

          /** \brief Evaluation of the derivative with respect to u at the (u,v) parameters */
          double diff_u(const double& u,const double& v) const;
          /** \brief Evaluation of the derivative with respect to v at the (u,v) parameters */
          double diff_v(const double& u,const double& v) const;

      private:

          /** \brief Internal storage for the patch values */
          cpe::matrix4 P;

  };
}

#endif
