/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <surface_param.hpp>
#include <string_helper.hpp>


namespace cpe
{
surface_param::surface_param()
    :internal_size_u(0),internal_size_v(0)
{}

surface_param::surface_param(const unsigned int& Nu,const unsigned int& Nv)
    :internal_size_u(Nu),internal_size_v(Nv)
{
    internal_data.resize(Nu*Nv);
}

void surface_param::resize(const unsigned int& Nu,const unsigned int& Nv)
{
    internal_size_u=Nu;
    internal_size_v=Nv;
    internal_data.resize(Nu*Nv);
}

const unsigned int& surface_param::size_u() const
{
    return internal_size_u;
}

const unsigned int& surface_param::size_v() const
{
    return internal_size_v;
}

const v3& surface_param::operator()(const unsigned int& ku,const unsigned int& kv) const
{
    assert_size(ku,kv);
    return internal_data[ku+internal_size_u*kv];
}

v3& surface_param::operator()(const unsigned int& ku,const unsigned int& kv)
{
    assert_size(ku,kv);
    return internal_data[ku+internal_size_u*kv];
}

const v3* surface_param::pointer() const
{
    return &internal_data[0];
}
v3* surface_param::pointer_unprotected()
{
    return &internal_data[0];
}

void surface_param::assert_size(const unsigned int& ku,const unsigned int& kv) const
{
    if(ku>=internal_size_u || kv>=internal_size_v)
    {
        throw std::string("Exception ")+std::string(__PRETTY_FUNCTION__)+" ("+cpe::string_converter::to_string(ku)+","+cpe::string_converter::to_string(kv)+")>("+cpe::string_converter::to_string(internal_size_u)+","+cpe::string_converter::to_string(internal_size_v)+")";
    }
}

const v3& surface_param::get(const unsigned int& ku,const unsigned int& kv) const
{
    //warning unprotected access
    return internal_data[ku+internal_size_u*kv];
}




}
