/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _QUAD_INDEX_HPP_
#define _QUAD_INDEX_HPP_

namespace cpe
{
class quad_index
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        quad_index();
        /** \brief direct constructor */
        quad_index(const unsigned int& u0,const unsigned int& u1,const unsigned int& u2,const unsigned int& u3);


        // ********************************************* //
        // ********************************************* //
        //  Data access
        // ********************************************* //
        // ********************************************* //

        /** \brief Access first component */
        const unsigned int& u0() const;
        /** \brief Access second component */
        const unsigned int& u1() const;
        /** \brief Access third component */
        const unsigned int& u2() const;
        /** \brief Access fourth component */
        const unsigned int& u3() const;

        /** \brief Access first component */
        unsigned int& u0();
        /** \brief Access second component */
        unsigned int& u1();
        /** \brief Access third component */
        unsigned int& u2();
        /** \brief Access fourth component */
        unsigned int& u3();

        /** \brief Access component k (0<=k<4) component */
        const unsigned int& operator[](const unsigned int& k_index) const;
        /** \brief Access component k (0<=k<4) component */
        unsigned int& operator[](const unsigned int& k_index);

    private:

        /** \brief assert that a size_t belongs to [[0,3]] */
        void assert_size(const unsigned int& k) const;

        /** \brief internal storage of 4 components = indices of the quad */
        unsigned int internal_u[4];
};
}

#endif
