/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PATCH4_HPP_
#define _PATCH4_HPP_

#include <v3.hpp>

namespace cpe
{
class matrix4;

class patch4
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief Empty constructor */
        patch4();
        /** \brief Direct constructor */
        patch4(const v3& x00,const v3& x01,const v3& x02,const v3& x03,
               const v3& x10,const v3& x11,const v3& x12,const v3& x13,
               const v3& x20,const v3& x21,const v3& x22,const v3& x23,
               const v3& x30,const v3& x31,const v3& x32,const v3& x33);

        // ********************************************* //
        // ********************************************* //
        //  Data
        // ********************************************* //
        // ********************************************* //

        /** \brief Get value */
        const v3& operator()(const unsigned int& ku ,const unsigned int& kv) const;
        /** \brief Get value */
        v3& operator()(const unsigned int& ku ,const unsigned int& kv);

        /** \brief Get x coordinate in matrix4 form */
        matrix4 get_x() const;
        /** \brief Get y coordinate in matrix4 form */
        matrix4 get_y() const;
        /** \brief Get z coordinate in matrix4 form */
        matrix4 get_z() const;

    private:

        /** Ensure (ku,kv) is within [[0,3]] */
        void assert_size(const unsigned int& ku,const unsigned int& kv) const;

        /** \brief internal data */
        v3 internal_data[16];
};
}

#endif
