/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _SURFACE_PARAM_HPP_
#define _SURFACE_PARAM_HPP_

#include <v3.hpp>
#include <vector>

namespace cpe
{
class surface_param
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        surface_param();
        /** \brief direct constructor */
        surface_param(const unsigned int& Nu,const unsigned int& Nv);

        // ********************************************* //
        // ********************************************* //
        //  Size
        // ********************************************* //
        // ********************************************* //


        /** \brief resize the grid */
        void resize(const unsigned int& Nu,const unsigned int& Nv);

        /** \brief return the size in u direction */
        const unsigned int& size_u() const;
        /** \brief return the size in v direction */
        const unsigned int& size_v() const;


        // ********************************************* //
        // ********************************************* //
        //  Data
        // ********************************************* //
        // ********************************************* //

        /** \brief Accessor to the value (ku,kv) */
        const v3& operator()(const unsigned int& ku,const unsigned int& kv) const;
        /** \brief Accessor to the value (ku,kv) */
        v3& operator()(const unsigned int& ku,const unsigned int& kv);


        // ********************************************* //
        // ********************************************* //
        //  Internal access
        // ********************************************* //
        // ********************************************* //

        /** \brief export internal pointer */
        const v3* pointer() const;
        /** \brief export internal pointer unprotected, use with care */
        v3* pointer_unprotected();


    protected:

        /** \brief getter fast but unprotected for heritage */
        const v3& get(const unsigned int& ku,const unsigned int& kv) const;

        /** \brief ensure (ku,kv) is coherent with the size of the grid */
        void assert_size(const unsigned int& ku,const unsigned int& kv) const;

    private:





        /** \brief internal size in u */
        unsigned int internal_size_u;
        /** \brief internal size in v */
        unsigned int internal_size_v;
        /** \brief internal data */
        std::vector<v3> internal_data;
};
}

#endif
