/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _SURFACE_PARAM_DRAWABLE_HPP_
#define _SURFACE_PARAM_DRAWABLE_HPP_

#include <surface_param.hpp>
#include <v3.hpp>
#include <quad_index.hpp>

namespace cpe
{



/** \brief Wrapper class to help drawing a surface_param class */
class surface_param_drawable : public surface_param
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        surface_param_drawable();
        /** \brief direct constructor with size */
        surface_param_drawable(const unsigned int& Nu,const unsigned int& Nv);

        // ********************************************* //
        // ********************************************* //
        //  Update drawing parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief update connectivity of needed */
        void update_connectivity();
        /** \bried update normal */
        void update_normal();

        /** \brief set a specific normal */
        void set_normal(const unsigned int ku,const unsigned int kv,const cpe::v3 n);


        // ********************************************* //
        // ********************************************* //
        //  Get drawing pointers
        // ********************************************* //
        // ********************************************* //

        /** \brief get pointer on connectivity */
        const unsigned int* pointer_draw_connectivity() const;
        /** \brief get pointer on normal */
        const double* pointer_draw_normal() const;
        /** \brief get pointer on vertices */
        const double* pointer_draw_vertices() const;





    private:

        /** \brief force to build connectivity */
        void build_connectivity();

        /** \brief internal connectivity */
        std::vector<quad_index> internal_connectivity;
        /** \brief internal normal */
        std::vector<v3> internal_normal;

        /** \brief storage for size u stored since the previous update */
        unsigned int internal_old_size_u;
        /** \brief storage for size u stored since the previous update */
        unsigned int internal_old_size_v;
};
}

#endif
